@@ -12,6 +12,7 @@ from PIL import Image, ImageSequence, PngImagePlugin
 # (referenced from https://wiki.mozilla.org/APNG_Specification)
 def test_apng_basic() -> None:
     with Image.open("Tests/images/apng/single_frame.png") as im:
+        assert isinstance(im, PngImagePlugin.PngImageFile)
         assert not im.is_animated
         assert im.n_frames == 1
         assert im.get_format_mimetype() == "image/apng"
@@ -20,6 +21,7 @@ def test_apng_basic() -> None:
         assert im.getpixel((64, 32)) == (0, 255, 0, 255)
 
     with Image.open("Tests/images/apng/single_frame_default.png") as im:
+        assert isinstance(im, PngImagePlugin.PngImageFile)
         assert im.is_animated
         assert im.n_frames == 2
         assert im.get_format_mimetype() == "image/apng"
@@ -34,8 +36,11 @@ def test_apng_basic() -> None:
         with pytest.raises(EOFError):
             im.seek(2)
 
-        # test rewind support
         im.seek(0)
+        with pytest.raises(ValueError, match="cannot seek to frame 2"):
+            im._seek(2)
+
+        # test rewind support
         assert im.getpixel((0, 0)) == (255, 0, 0, 255)
         assert im.getpixel((64, 32)) == (255, 0, 0, 255)
         im.seek(1)
@@ -49,6 +54,7 @@ def test_apng_basic() -> None:
 )
 def test_apng_fdat(filename: str) -> None:
     with Image.open(filename) as im:
+        assert isinstance(im, PngImagePlugin.PngImageFile)
         im.seek(im.n_frames - 1)
         assert im.getpixel((0, 0)) == (0, 255, 0, 255)
         assert im.getpixel((64, 32)) == (0, 255, 0, 255)
@@ -56,31 +62,37 @@ def test_apng_fdat(filename: str) -> None:
 
 def test_apng_dispose() -> None:
     with Image.open("Tests/images/apng/dispose_op_none.png") as im:
+        assert isinstance(im, PngImagePlugin.PngImageFile)
         im.seek(im.n_frames - 1)
         assert im.getpixel((0, 0)) == (0, 255, 0, 255)
         assert im.getpixel((64, 32)) == (0, 255, 0, 255)
 
     with Image.open("Tests/images/apng/dispose_op_background.png") as im:
+        assert isinstance(im, PngImagePlugin.PngImageFile)
         im.seek(im.n_frames - 1)
         assert im.getpixel((0, 0)) == (0, 0, 0, 0)
         assert im.getpixel((64, 32)) == (0, 0, 0, 0)
 
     with Image.open("Tests/images/apng/dispose_op_background_final.png") as im:
+        assert isinstance(im, PngImagePlugin.PngImageFile)
         im.seek(im.n_frames - 1)
         assert im.getpixel((0, 0)) == (0, 255, 0, 255)
         assert im.getpixel((64, 32)) == (0, 255, 0, 255)
 
     with Image.open("Tests/images/apng/dispose_op_previous.png") as im:
+        assert isinstance(im, PngImagePlugin.PngImageFile)
         im.seek(im.n_frames - 1)
         assert im.getpixel((0, 0)) == (0, 255, 0, 255)
         assert im.getpixel((64, 32)) == (0, 255, 0, 255)
 
     with Image.open("Tests/images/apng/dispose_op_previous_final.png") as im:
+        assert isinstance(im, PngImagePlugin.PngImageFile)
         im.seek(im.n_frames - 1)
         assert im.getpixel((0, 0)) == (0, 255, 0, 255)
         assert im.getpixel((64, 32)) == (0, 255, 0, 255)
 
     with Image.open("Tests/images/apng/dispose_op_previous_first.png") as im:
+        assert isinstance(im, PngImagePlugin.PngImageFile)
         im.seek(im.n_frames - 1)
         assert im.getpixel((0, 0)) == (0, 0, 0, 0)
         assert im.getpixel((64, 32)) == (0, 0, 0, 0)
@@ -88,21 +100,25 @@ def test_apng_dispose() -> None:
 
 def test_apng_dispose_region() -> None:
     with Image.open("Tests/images/apng/dispose_op_none_region.png") as im:
+        assert isinstance(im, PngImagePlugin.PngImageFile)
         im.seek(im.n_frames - 1)
         assert im.getpixel((0, 0)) == (0, 255, 0, 255)
         assert im.getpixel((64, 32)) == (0, 255, 0, 255)
 
     with Image.open("Tests/images/apng/dispose_op_background_before_region.png") as im:
+        assert isinstance(im, PngImagePlugin.PngImageFile)
         im.seek(im.n_frames - 1)
         assert im.getpixel((0, 0)) == (0, 0, 0, 0)
         assert im.getpixel((64, 32)) == (0, 0, 0, 0)
 
     with Image.open("Tests/images/apng/dispose_op_background_region.png") as im:
+        assert isinstance(im, PngImagePlugin.PngImageFile)
         im.seek(im.n_frames - 1)
         assert im.getpixel((0, 0)) == (0, 0, 255, 255)
         assert im.getpixel((64, 32)) == (0, 0, 0, 0)
 
     with Image.open("Tests/images/apng/dispose_op_previous_region.png") as im:
+        assert isinstance(im, PngImagePlugin.PngImageFile)
         im.seek(im.n_frames - 1)
         assert im.getpixel((0, 0)) == (0, 255, 0, 255)
         assert im.getpixel((64, 32)) == (0, 255, 0, 255)
@@ -129,6 +145,7 @@ def test_apng_dispose_op_previous_frame() -> None:
     #     ],
     # )
     with Image.open("Tests/images/apng/dispose_op_previous_frame.png") as im:
+        assert isinstance(im, PngImagePlugin.PngImageFile)
         im.seek(im.n_frames - 1)
         assert im.getpixel((0, 0)) == (255, 0, 0, 255)
 
@@ -142,26 +159,31 @@ def test_apng_dispose_op_background_p_mode() -> None:
 
 def test_apng_blend() -> None:
     with Image.open("Tests/images/apng/blend_op_source_solid.png") as im:
+        assert isinstance(im, PngImagePlugin.PngImageFile)
         im.seek(im.n_frames - 1)
         assert im.getpixel((0, 0)) == (0, 255, 0, 255)
         assert im.getpixel((64, 32)) == (0, 255, 0, 255)
 
     with Image.open("Tests/images/apng/blend_op_source_transparent.png") as im:
+        assert isinstance(im, PngImagePlugin.PngImageFile)
         im.seek(im.n_frames - 1)
         assert im.getpixel((0, 0)) == (0, 0, 0, 0)
         assert im.getpixel((64, 32)) == (0, 0, 0, 0)
 
     with Image.open("Tests/images/apng/blend_op_source_near_transparent.png") as im:
+        assert isinstance(im, PngImagePlugin.PngImageFile)
         im.seek(im.n_frames - 1)
         assert im.getpixel((0, 0)) == (0, 255, 0, 2)
         assert im.getpixel((64, 32)) == (0, 255, 0, 2)
 
     with Image.open("Tests/images/apng/blend_op_over.png") as im:
+        assert isinstance(im, PngImagePlugin.PngImageFile)
         im.seek(im.n_frames - 1)
         assert im.getpixel((0, 0)) == (0, 255, 0, 255)
         assert im.getpixel((64, 32)) == (0, 255, 0, 255)
 
     with Image.open("Tests/images/apng/blend_op_over_near_transparent.png") as im:
+        assert isinstance(im, PngImagePlugin.PngImageFile)
         im.seek(im.n_frames - 1)
         assert im.getpixel((0, 0)) == (0, 255, 0, 97)
         assert im.getpixel((64, 32)) == (0, 255, 0, 255)
@@ -175,6 +197,7 @@ def test_apng_blend_transparency() -> None:
 
 def test_apng_chunk_order() -> None:
     with Image.open("Tests/images/apng/fctl_actl.png") as im:
+        assert isinstance(im, PngImagePlugin.PngImageFile)
         im.seek(im.n_frames - 1)
         assert im.getpixel((0, 0)) == (0, 255, 0, 255)
         assert im.getpixel((64, 32)) == (0, 255, 0, 255)
@@ -230,24 +253,28 @@ def test_apng_num_plays() -> None:
 
 def test_apng_mode() -> None:
     with Image.open("Tests/images/apng/mode_16bit.png") as im:
+        assert isinstance(im, PngImagePlugin.PngImageFile)
         assert im.mode == "RGBA"
         im.seek(im.n_frames - 1)
         assert im.getpixel((0, 0)) == (0, 0, 128, 191)
         assert im.getpixel((64, 32)) == (0, 0, 128, 191)
 
     with Image.open("Tests/images/apng/mode_grayscale.png") as im:
+        assert isinstance(im, PngImagePlugin.PngImageFile)
         assert im.mode == "L"
         im.seek(im.n_frames - 1)
         assert im.getpixel((0, 0)) == 128
         assert im.getpixel((64, 32)) == 255
 
     with Image.open("Tests/images/apng/mode_grayscale_alpha.png") as im:
+        assert isinstance(im, PngImagePlugin.PngImageFile)
         assert im.mode == "LA"
         im.seek(im.n_frames - 1)
         assert im.getpixel((0, 0)) == (128, 191)
         assert im.getpixel((64, 32)) == (128, 191)
 
     with Image.open("Tests/images/apng/mode_palette.png") as im:
+        assert isinstance(im, PngImagePlugin.PngImageFile)
         assert im.mode == "P"
         im.seek(im.n_frames - 1)
         im = im.convert("RGB")
@@ -255,13 +282,15 @@ def test_apng_mode() -> None:
         assert im.getpixel((64, 32)) == (0, 255, 0)
 
     with Image.open("Tests/images/apng/mode_palette_alpha.png") as im:
+        assert isinstance(im, PngImagePlugin.PngImageFile)
         assert im.mode == "P"
         im.seek(im.n_frames - 1)
         im = im.convert("RGBA")
-        assert im.getpixel((0, 0)) == (255, 0, 0, 0)
-        assert im.getpixel((64, 32)) == (255, 0, 0, 0)
+        assert im.getpixel((0, 0)) == (0, 255, 0, 255)
+        assert im.getpixel((64, 32)) == (0, 255, 0, 255)
 
     with Image.open("Tests/images/apng/mode_palette_1bit_alpha.png") as im:
+        assert isinstance(im, PngImagePlugin.PngImageFile)
         assert im.mode == "P"
         im.seek(im.n_frames - 1)
         im = im.convert("RGBA")
@@ -271,25 +300,31 @@ def test_apng_mode() -> None:
 
 def test_apng_chunk_errors() -> None:
     with Image.open("Tests/images/apng/chunk_no_actl.png") as im:
+        assert isinstance(im, PngImagePlugin.PngImageFile)
         assert not im.is_animated
 
     with pytest.warns(UserWarning):
         with Image.open("Tests/images/apng/chunk_multi_actl.png") as im:
             im.load()
+        assert isinstance(im, PngImagePlugin.PngImageFile)
         assert not im.is_animated
 
     with Image.open("Tests/images/apng/chunk_actl_after_idat.png") as im:
+        assert isinstance(im, PngImagePlugin.PngImageFile)
         assert not im.is_animated
 
     with Image.open("Tests/images/apng/chunk_no_fctl.png") as im:
+        assert isinstance(im, PngImagePlugin.PngImageFile)
         with pytest.raises(SyntaxError):
             im.seek(im.n_frames - 1)
 
     with Image.open("Tests/images/apng/chunk_repeat_fctl.png") as im:
+        assert isinstance(im, PngImagePlugin.PngImageFile)
         with pytest.raises(SyntaxError):
             im.seek(im.n_frames - 1)
 
     with Image.open("Tests/images/apng/chunk_no_fdat.png") as im:
+        assert isinstance(im, PngImagePlugin.PngImageFile)
         with pytest.raises(SyntaxError):
             im.seek(im.n_frames - 1)
 
@@ -297,31 +332,31 @@ def test_apng_chunk_errors() -> None:
 def test_apng_syntax_errors() -> None:
     with pytest.warns(UserWarning):
         with Image.open("Tests/images/apng/syntax_num_frames_zero.png") as im:
+            assert isinstance(im, PngImagePlugin.PngImageFile)
             assert not im.is_animated
             with pytest.raises(OSError):
                 im.load()
 
     with pytest.warns(UserWarning):
         with Image.open("Tests/images/apng/syntax_num_frames_zero_default.png") as im:
+            assert isinstance(im, PngImagePlugin.PngImageFile)
             assert not im.is_animated
             im.load()
 
     # we can handle this case gracefully
-    exception = None
     with Image.open("Tests/images/apng/syntax_num_frames_low.png") as im:
-        try:
-            im.seek(im.n_frames - 1)
-        except Exception as e:
-            exception = e
-        assert exception is None
+        assert isinstance(im, PngImagePlugin.PngImageFile)
+        im.seek(im.n_frames - 1)
 
     with pytest.raises(OSError):
         with Image.open("Tests/images/apng/syntax_num_frames_high.png") as im:
+            assert isinstance(im, PngImagePlugin.PngImageFile)
             im.seek(im.n_frames - 1)
             im.load()
 
     with pytest.warns(UserWarning):
         with Image.open("Tests/images/apng/syntax_num_frames_invalid.png") as im:
+            assert isinstance(im, PngImagePlugin.PngImageFile)
             assert not im.is_animated
             im.load()
 
@@ -341,16 +376,18 @@ def test_apng_syntax_errors() -> None:
 def test_apng_sequence_errors(test_file: str) -> None:
     with pytest.raises(SyntaxError):
         with Image.open(f"Tests/images/apng/{test_file}") as im:
+            assert isinstance(im, PngImagePlugin.PngImageFile)
             im.seek(im.n_frames - 1)
             im.load()
 
 
 def test_apng_save(tmp_path: Path) -> None:
     with Image.open("Tests/images/apng/single_frame.png") as im:
-        test_file = str(tmp_path / "temp.png")
+        test_file = tmp_path / "temp.png"
         im.save(test_file, save_all=True)
 
     with Image.open(test_file) as im:
+        assert isinstance(im, PngImagePlugin.PngImageFile)
         im.load()
         assert not im.is_animated
         assert im.n_frames == 1
@@ -366,6 +403,7 @@ def test_apng_save(tmp_path: Path) -> None:
         )
 
     with Image.open(test_file) as im:
+        assert isinstance(im, PngImagePlugin.PngImageFile)
         im.load()
         assert im.is_animated
         assert im.n_frames == 2
@@ -377,7 +415,7 @@ def test_apng_save(tmp_path: Path) -> None:
 
 
 def test_apng_save_alpha(tmp_path: Path) -> None:
-    test_file = str(tmp_path / "temp.png")
+    test_file = tmp_path / "temp.png"
 
     im = Image.new("RGBA", (1, 1), (255, 0, 0, 255))
     im2 = Image.new("RGBA", (1, 1), (255, 0, 0, 127))
@@ -395,7 +433,7 @@ def test_apng_save_split_fdat(tmp_path: Path) -> None:
     # frames with image data spanning multiple fdAT chunks (in this case
     # both the default image and first animation frame will span multiple
     # data chunks)
-    test_file = str(tmp_path / "temp.png")
+    test_file = tmp_path / "temp.png"
     with Image.open("Tests/images/old-style-jpeg-compression.png") as im:
         frames = [im.copy(), Image.new("RGBA", im.size, (255, 0, 0, 255))]
         im.save(
@@ -405,17 +443,13 @@ def test_apng_save_split_fdat(tmp_path: Path) -> None:
             append_images=frames,
         )
     with Image.open(test_file) as im:
-        exception = None
-        try:
-            im.seek(im.n_frames - 1)
-            im.load()
-        except Exception as e:
-            exception = e
-        assert exception is None
+        assert isinstance(im, PngImagePlugin.PngImageFile)
+        im.seek(im.n_frames - 1)
+        im.load()
 
 
 def test_apng_save_duration_loop(tmp_path: Path) -> None:
-    test_file = str(tmp_path / "temp.png")
+    test_file = tmp_path / "temp.png"
     with Image.open("Tests/images/apng/delay.png") as im:
         frames = []
         durations = []
@@ -452,6 +486,7 @@ def test_apng_save_duration_loop(tmp_path: Path) -> None:
         test_file, save_all=True, append_images=[frame, frame], duration=[500, 100, 150]
     )
     with Image.open(test_file) as im:
+        assert isinstance(im, PngImagePlugin.PngImageFile)
         assert im.n_frames == 1
         assert "duration" not in im.info
 
@@ -463,6 +498,7 @@ def test_apng_save_duration_loop(tmp_path: Path) -> None:
         duration=[500, 100, 150],
     )
     with Image.open(test_file) as im:
+        assert isinstance(im, PngImagePlugin.PngImageFile)
         assert im.n_frames == 2
         assert im.info["duration"] == 600
 
@@ -473,12 +509,13 @@ def test_apng_save_duration_loop(tmp_path: Path) -> None:
     frame.info["duration"] = 300
     frame.save(test_file, save_all=True, append_images=[frame, different_frame])
     with Image.open(test_file) as im:
+        assert isinstance(im, PngImagePlugin.PngImageFile)
         assert im.n_frames == 2
         assert im.info["duration"] == 600
 
 
 def test_apng_save_disposal(tmp_path: Path) -> None:
-    test_file = str(tmp_path / "temp.png")
+    test_file = tmp_path / "temp.png"
     size = (128, 64)
     red = Image.new("RGBA", size, (255, 0, 0, 255))
     green = Image.new("RGBA", size, (0, 255, 0, 255))
@@ -579,7 +616,7 @@ def test_apng_save_disposal(tmp_path: Path) -> None:
 
 
 def test_apng_save_disposal_previous(tmp_path: Path) -> None:
-    test_file = str(tmp_path / "temp.png")
+    test_file = tmp_path / "temp.png"
     size = (128, 64)
     blue = Image.new("RGBA", size, (0, 0, 255, 255))
     red = Image.new("RGBA", size, (255, 0, 0, 255))
@@ -601,7 +638,7 @@ def test_apng_save_disposal_previous(tmp_path: Path) -> None:
 
 
 def test_apng_save_blend(tmp_path: Path) -> None:
-    test_file = str(tmp_path / "temp.png")
+    test_file = tmp_path / "temp.png"
     size = (128, 64)
     red = Image.new("RGBA", size, (255, 0, 0, 255))
     green = Image.new("RGBA", size, (0, 255, 0, 255))
@@ -669,7 +706,7 @@ def test_apng_save_blend(tmp_path: Path) -> None:
 
 
 def test_apng_save_size(tmp_path: Path) -> None:
-    test_file = str(tmp_path / "temp.png")
+    test_file = tmp_path / "temp.png"
 
     im = Image.new("L", (100, 100))
     im.save(test_file, save_all=True, append_images=[Image.new("L", (200, 200))])
@@ -693,7 +730,7 @@ def test_seek_after_close() -> None:
 def test_different_modes_in_later_frames(
     mode: str, default_image: bool, duplicate: bool, tmp_path: Path
 ) -> None:
-    test_file = str(tmp_path / "temp.png")
+    test_file = tmp_path / "temp.png"
 
     im = Image.new("L", (1, 1))
     im.save(
@@ -706,10 +743,21 @@ def test_different_modes_in_later_frames(
         assert reloaded.mode == mode
 
 
-def test_apng_repeated_seeks_give_correct_info() -> None:
+def test_different_durations(tmp_path: Path) -> None:
+    test_file = tmp_path / "temp.png"
+
     with Image.open("Tests/images/apng/different_durations.png") as im:
-        for i in range(3):
+        for _ in range(3):
             im.seek(0)
             assert im.info["duration"] == 4000
+
             im.seek(1)
             assert im.info["duration"] == 1000
+
+        im.save(test_file, save_all=True)
+
+    with Image.open(test_file) as reloaded:
+        assert reloaded.info["duration"] == 4000
+
+        reloaded.seek(1)
+        assert reloaded.info["duration"] == 1000
