@@ -1,7 +1,8 @@
 from __future__ import annotations
 
-import contextlib
 import os.path
+from collections.abc import Sequence
+from typing import Callable
 
 import pytest
 
@@ -38,6 +39,8 @@ BBOX = (((X0, Y0), (X1, Y1)), [(X0, Y0), (X1, Y1)], (X0, Y0, X1, Y1), [X0, Y0, X
 POINTS = (
     ((10, 10), (20, 40), (30, 30)),
     [(10, 10), (20, 40), (30, 30)],
+    ([10, 10], [20, 40], [30, 30]),
+    [[10, 10], [20, 40], [30, 30]],
     (10, 10, 20, 40, 30, 30),
     [10, 10, 20, 40, 30, 30],
 )
@@ -45,6 +48,8 @@ POINTS = (
 KITE_POINTS = (
     ((10, 50), (70, 10), (90, 50), (70, 90), (10, 50)),
     [(10, 50), (70, 10), (90, 50), (70, 90), (10, 50)],
+    ([10, 50], [70, 10], [90, 50], [70, 90], [10, 50]),
+    [[10, 50], [70, 10], [90, 50], [70, 90], [10, 50]],
 )
 
 
@@ -265,6 +270,21 @@ def test_chord_too_fat() -> None:
     assert_image_equal_tofile(im, "Tests/images/imagedraw_chord_too_fat.png")
 
 
+@pytest.mark.parametrize("mode", ("RGB", "L"))
+@pytest.mark.parametrize("xy", ((W / 2, H / 2), [W / 2, H / 2]))
+def test_circle(mode: str, xy: Sequence[float]) -> None:
+    # Arrange
+    im = Image.new(mode, (W, H))
+    draw = ImageDraw.Draw(im)
+    expected = f"Tests/images/imagedraw_ellipse_{mode}.png"
+
+    # Act
+    draw.circle(xy, 25, fill="green", outline="blue")
+
+    # Assert
+    assert_image_similar_tofile(im, expected, 1)
+
+
 @pytest.mark.parametrize("mode", ("RGB", "L"))
 @pytest.mark.parametrize("bbox", BBOX)
 def test_ellipse(mode: str, bbox: Coords) -> None:
@@ -615,6 +635,19 @@ def test_polygon(points: Coords) -> None:
     assert_image_equal_tofile(im, "Tests/images/imagedraw_polygon.png")
 
 
+@pytest.mark.parametrize("points", POINTS)
+def test_polygon_width_I16(points: Coords) -> None:
+    # Arrange
+    im = Image.new("I;16", (W, H))
+    draw = ImageDraw.Draw(im)
+
+    # Act
+    draw.polygon(points, outline=0xFFFF, width=2)
+
+    # Assert
+    assert_image_equal_tofile(im, "Tests/images/imagedraw_polygon_width_I.tiff")
+
+
 @pytest.mark.parametrize("mode", ("RGB", "L"))
 @pytest.mark.parametrize("kite_points", KITE_POINTS)
 def test_polygon_kite(
@@ -780,7 +813,7 @@ def test_rounded_rectangle(
         tuple[int, int, int, int]
         | tuple[list[int]]
         | tuple[tuple[int, int], tuple[int, int]]
-    )
+    ),
 ) -> None:
     # Arrange
     im = Image.new("RGB", (200, 200))
@@ -825,6 +858,27 @@ def test_rounded_rectangle_corners(
     )
 
 
+def test_rounded_rectangle_joined_x_different_corners() -> None:
+    # Arrange
+    im = Image.new("RGB", (W, H))
+    draw = ImageDraw.Draw(im, "RGBA")
+
+    # Act
+    draw.rounded_rectangle(
+        (20, 10, 80, 90),
+        30,
+        fill="red",
+        outline="green",
+        width=5,
+        corners=(True, False, False, False),
+    )
+
+    # Assert
+    assert_image_equal_tofile(
+        im, "Tests/images/imagedraw_rounded_rectangle_joined_x_different_corners.png"
+    )
+
+
 @pytest.mark.parametrize(
     "xy, radius, type",
     [
@@ -897,7 +951,12 @@ def test_rounded_rectangle_translucent(
 def test_floodfill(bbox: Coords) -> None:
     red = ImageColor.getrgb("red")
 
-    for mode, value in [("L", 1), ("RGBA", (255, 0, 0, 0)), ("RGB", red)]:
+    mode_values: list[tuple[str, int | tuple[int, ...]]] = [
+        ("L", 1),
+        ("RGBA", (255, 0, 0, 0)),
+        ("RGB", red),
+    ]
+    for mode, value in mode_values:
         # Arrange
         im = Image.new(mode, (W, H))
         draw = ImageDraw.Draw(im)
@@ -989,8 +1048,8 @@ def create_base_image_draw(
     background2: tuple[int, int, int] = GRAY,
 ) -> tuple[Image.Image, ImageDraw.ImageDraw]:
     img = Image.new(mode, size, background1)
-    for x in range(0, size[0]):
-        for y in range(0, size[1]):
+    for x in range(size[0]):
+        for y in range(size[1]):
             if (x + y) % 2 == 0:
                 img.putpixel((x, y), background2)
     return img, ImageDraw.Draw(img)
@@ -1067,8 +1126,8 @@ def test_line_horizontal() -> None:
     )
 
 
+@pytest.mark.xfail(reason="failing test")
 def test_line_h_s1_w2() -> None:
-    pytest.skip("failing")
     img, draw = create_base_image_draw((20, 20))
     draw.line((5, 5, 14, 6), BLACK, 2)
     assert_image_equal_tofile(
@@ -1310,6 +1369,20 @@ def test_stroke() -> None:
         )
 
 
+@skip_unless_feature("freetype2")
+def test_stroke_float() -> None:
+    # Arrange
+    im = Image.new("RGB", (120, 130))
+    draw = ImageDraw.Draw(im)
+    font = ImageFont.truetype("Tests/fonts/FreeMono.ttf", 120)
+
+    # Act
+    draw.text((12, 12), "A", "#f00", font, stroke_width=0.5)
+
+    # Assert
+    assert_image_similar_tofile(im, "Tests/images/imagedraw_stroke_float.png", 3.1)
+
+
 @skip_unless_feature("freetype2")
 def test_stroke_descender() -> None:
     # Arrange
@@ -1324,6 +1397,28 @@ def test_stroke_descender() -> None:
     assert_image_similar_tofile(im, "Tests/images/imagedraw_stroke_descender.png", 6.76)
 
 
+@skip_unless_feature("freetype2")
+def test_stroke_inside_gap() -> None:
+    # Arrange
+    im = Image.new("RGB", (120, 130))
+    draw = ImageDraw.Draw(im)
+    font = ImageFont.truetype("Tests/fonts/FreeMono.ttf", 120)
+
+    # Act
+    draw.text((12, 12), "i", "#f00", font, stroke_width=20)
+
+    # Assert
+    for y in range(im.height):
+        glyph = ""
+        for x in range(im.width):
+            if im.getpixel((x, y)) == (0, 0, 0):
+                if glyph == "started":
+                    glyph = "ended"
+            else:
+                assert glyph != "ended", "Gap inside stroked glyph"
+                glyph = "started"
+
+
 @skip_unless_feature("freetype2")
 def test_split_word() -> None:
     # Arrange
@@ -1385,25 +1480,44 @@ def test_default_font_size() -> None:
 
     im = Image.new("RGB", (220, 25))
     draw = ImageDraw.Draw(im)
-    with contextlib.nullcontext() if freetype_support else pytest.raises(ImportError):
+
+    def check(func: Callable[[], None]) -> None:
+        if freetype_support:
+            func()
+        else:
+            with pytest.raises(ImportError):
+                func()
+
+    def draw_text() -> None:
         draw.text((0, 0), text, font_size=16)
         assert_image_equal_tofile(im, "Tests/images/imagedraw_default_font_size.png")
 
-    with contextlib.nullcontext() if freetype_support else pytest.raises(ImportError):
+    check(draw_text)
+
+    def draw_textlength() -> None:
         assert draw.textlength(text, font_size=16) == 216
 
-    with contextlib.nullcontext() if freetype_support else pytest.raises(ImportError):
+    check(draw_textlength)
+
+    def draw_textbbox() -> None:
         assert draw.textbbox((0, 0), text, font_size=16) == (0, 3, 216, 19)
 
+    check(draw_textbbox)
+
     im = Image.new("RGB", (220, 25))
     draw = ImageDraw.Draw(im)
-    with contextlib.nullcontext() if freetype_support else pytest.raises(ImportError):
+
+    def draw_multiline_text() -> None:
         draw.multiline_text((0, 0), text, font_size=16)
         assert_image_equal_tofile(im, "Tests/images/imagedraw_default_font_size.png")
 
-    with contextlib.nullcontext() if freetype_support else pytest.raises(ImportError):
+    check(draw_multiline_text)
+
+    def draw_multiline_textbbox() -> None:
         assert draw.multiline_textbbox((0, 0), text, font_size=16) == (0, 3, 216, 19)
 
+    check(draw_multiline_textbbox)
+
 
 @pytest.mark.parametrize("bbox", BBOX)
 def test_same_color_outline(bbox: Coords) -> None:
@@ -1451,7 +1565,7 @@ def test_same_color_outline(bbox: Coords) -> None:
         (4, "square", {}),
         (8, "regular_octagon", {}),
         (4, "square_rotate_45", {"rotation": 45}),
-        (3, "triangle_width", {"width": 5, "outline": "yellow"}),
+        (3, "triangle_width", {"outline": "yellow", "width": 5}),
     ],
 )
 def test_draw_regular_polygon(
@@ -1461,7 +1575,10 @@ def test_draw_regular_polygon(
     filename = f"Tests/images/imagedraw_{polygon_name}.png"
     draw = ImageDraw.Draw(im)
     bounding_circle = ((W // 2, H // 2), 25)
-    draw.regular_polygon(bounding_circle, n_sides, fill="red", **args)
+    rotation = int(args.get("rotation", 0))
+    outline = args.get("outline")
+    width = int(args.get("width", 1))
+    draw.regular_polygon(bounding_circle, n_sides, rotation, "red", outline, width)
     assert_image_equal_tofile(im, filename)
 
 
@@ -1513,7 +1630,7 @@ def test_compute_regular_polygon_vertices(
             0,
             ValueError,
             "bounding_circle should contain 2D coordinates "
-            "and a radius (e.g. (x, y, r) or ((x, y), r) )",
+            r"and a radius \(e.g. \(x, y, r\) or \(\(x, y\), r\) \)",
         ),
         (
             3,
@@ -1527,7 +1644,7 @@ def test_compute_regular_polygon_vertices(
             ((50, 50, 50), 25),
             0,
             ValueError,
-            "bounding_circle centre should contain 2D coordinates (e.g. (x, y))",
+            r"bounding_circle centre should contain 2D coordinates \(e.g. \(x, y\)\)",
         ),
         (
             3,
@@ -1546,11 +1663,14 @@ def test_compute_regular_polygon_vertices(
     ],
 )
 def test_compute_regular_polygon_vertices_input_error_handling(
-    n_sides, bounding_circle, rotation, expected_error, error_message
+    n_sides: int,
+    bounding_circle: int | tuple[int | tuple[int] | str, ...],
+    rotation: int | str,
+    expected_error: type[Exception],
+    error_message: str,
 ) -> None:
-    with pytest.raises(expected_error) as e:
-        ImageDraw._compute_regular_polygon_vertices(bounding_circle, n_sides, rotation)
-    assert str(e.value) == error_message
+    with pytest.raises(expected_error, match=error_message):
+        ImageDraw._compute_regular_polygon_vertices(bounding_circle, n_sides, rotation)  # type: ignore[arg-type]
 
 
 def test_continuous_horizontal_edges_polygon() -> None:
@@ -1575,13 +1695,16 @@ def test_continuous_horizontal_edges_polygon() -> None:
 def test_discontiguous_corners_polygon() -> None:
     img, draw = create_base_image_draw((84, 68))
     draw.polygon(((1, 21), (34, 4), (71, 1), (38, 18)), BLACK)
+    draw.polygon(
+        ((82, 29), (82, 26), (82, 24), (67, 22), (52, 29), (52, 15), (67, 22)), BLACK
+    )
     draw.polygon(((71, 44), (38, 27), (1, 24)), BLACK)
     draw.polygon(
         ((38, 66), (5, 49), (77, 49), (47, 66), (82, 63), (82, 47), (1, 47), (1, 63)),
         BLACK,
     )
     expected = os.path.join(IMAGES_PATH, "discontiguous_corners_polygon.png")
-    assert_image_similar_tofile(img, expected, 1)
+    assert_image_equal_tofile(img, expected)
 
 
 def test_polygon2() -> None:
@@ -1608,3 +1731,8 @@ def test_incorrectly_ordered_coordinates(xy: tuple[int, int, int, int]) -> None:
         draw.rectangle(xy)
     with pytest.raises(ValueError):
         draw.rounded_rectangle(xy)
+
+
+def test_getdraw() -> None:
+    with pytest.warns(DeprecationWarning):
+        ImageDraw.getdraw(None, [])
